@@ -1,11 +1,11 @@
-## ----, echo=FALSE--------------------------------------------------------
-df <- structure(list(Z = c(1e+05, 1e+05, 1e+05, 1e+05, 1e+05), time = c(2006, 
+## ---- echo=FALSE---------------------------------------------------------
+co2_df <- structure(list(Z = c(1e+05, 1e+05, 1e+05, 1e+05, 1e+05), time = c(2006, 
 2007, 2008, 2009, 2010), value = c(380.717267354329, 382.683990478516, 
 384.720212300618, 386.920120239258, 389.219032287598)), .Names = c("Z", 
 "time", "value"), row.names = c(NA, -5L), class = "data.frame")
-plot(df$time, df$value, type='b')
+plot(co2_df$time, co2_df$value, type='b')
 
-## ----, eval=FALSE, echo=FALSE--------------------------------------------
+## ---- eval=FALSE, echo=FALSE---------------------------------------------
 #  # Here's the short script that produced the outputs in this vignette.
 #  # (Because it uses very large CMIP5 data files, we can't run it as
 #  # part of the vignette itself.)
