@@ -39,7 +39,7 @@ test_that("checkTimePeriod correctly sees continuous files", {
     
     d <- checkTimePeriod(getFileInfo(path))
     expect_is(d, "data.frame")
-    expect_more_than(nrow(d), 2)     # should be several cases
+    expect_gt(nrow(d), 2)     # should be several cases
     expect_equal(ncol(d), 10)
     expect_true(all(d$allHere))
 })
