@@ -12,8 +12,7 @@ library(testthat)
 context("getFileInfo")
 
 test_that("getFileInfo handles bad input", {
-    expect_error(getFileInfo("does_not_exist"),"file.exists\\(path) is not TRUE")
-                                                            # path does not exist
+    expect_error(getFileInfo("does_not_exist"))             # path does not exist
     expect_error(getFileInfo(path=c("1","2")))              # multi-value path
     expect_error(getFileInfo(recursive=c(T,F)))             # multi-value recursive
     expect_error(getFileInfo(12))                           # non-character path
@@ -25,7 +24,6 @@ test_that("getFileInfo handles no input", {                 # no NetCDF files fo
     options(warn=-1)
     path <- ("testdata_none")
     expect_warning(getFileInfo(path),"No NetCDF files found")
-    expect_is(getFileInfo(path),"NULL")
     options(warn=w)
 })
 
@@ -34,7 +32,6 @@ test_that("getFileInfo handles non-CMIP5 NetCDFs", {        # improper NetCDF fi
     options(warn=-1)
     path <- ("testdata_badfilename")
     expect_warning(getFileInfo(path),"Unexpected")
-    expect_is(getFileInfo(path),"NULL")
     options(warn=w)
 })
 
@@ -45,7 +42,7 @@ test_that("getFileInfo handles annual NetCDFs", {
     d <- getFileInfo(path)
     d <- d[complete.cases(d),]
     expect_is(d,"data.frame")
-    expect_more_than(nrow(d),1)     # should be several
+    expect_gt(nrow(d),1)     # should be several
     expect_equal(ncol(d),9)
     expect_equal(d[1, "path"],path)
     expect_equal(d[1, "time"],"2171-2172")
@@ -58,7 +55,7 @@ test_that("getFileInfo handles monthly NetCDFs", {
     d <- getFileInfo(path)
     d <- d[complete.cases(d),]
     expect_is(d,"data.frame")
-    expect_more_than(nrow(d),3)     # should be several
+    expect_gt(nrow(d),3)     # should be several
     expect_equal(ncol(d),9)
     expect_equal(d[1,"path"],path)
 })
